//
//  HTTPService.swift
//  IoTConnect
//
//  Created by Devesh Mevada on 8/23/21.
//

import Foundation

class HTTPService: HTTPManagerDelegate {
    
    static let shared = HTTPService()
    let session = URLSession.shared
    
    func makeRequest(config: HTTPRequestConfig, success: @escaping (Data) -> (), failure: @escaping (Error) -> ()) {
        if (checkInternetAvailable()) {
            if let request = getUrlRequest(config: config) {
                let task = session.dataTask(with: request) { (data, response, error) in
                    self.handleResponse(config: config, data: data, response: response, error: error, success: success, failure: failure)
                }
                task.resume()
            } else {
                failure(EBVHttpErrors.unableToCreateRequest)
            }
        } else {
            // send Internet is not available
            failure( EBVHttpErrors.noInternetConnection)
        }
    }
    
    func makeRequestarray(config: HTTPRequestConfigArray, success: @escaping (Data) -> (), failure: @escaping (Error) -> ()) {
        if (checkInternetAvailable()) {
            if let request = getUrlRequestarray(config: config) {
                let task = session.dataTask(with: request) { (data, response, error) in
                    self.handleResponsearray(config: config, data: data, response: response, error: error, success: success, failure: failure)
                }
                task.resume()
            } else {
                failure(EBVHttpErrors.unableToCreateRequest)
            }
        } else {
            // send Internet is not available
            failure( EBVHttpErrors.noInternetConnection)
        }
    }

    
    func makeRequestwithoutBody(config: HTTPRequestConfigwithoutBody, success: @escaping (Data) -> (), failure: @escaping (Error) -> ()) {
        if (checkInternetAvailable()) {
            if let request = getUrlRequestwithoutBody(config: config) {
                let task = session.dataTask(with: request) { (data, response, error) in
                    self.handleResponsewithoutBody(config: config, data: data, response: response, error: error, success: success, failure: failure)
                }
                task.resume()
            } else {
                failure(EBVHttpErrors.unableToCreateRequest)
            }
        } else {
            // send Internet is not available
            failure( EBVHttpErrors.noInternetConnection)
        }
    }

    
    func makeRequestForMultiPart(config: HTTPRequestConfig, success: @escaping (Data) -> (), failure: @escaping (Error) -> ()) {
        if (checkInternetAvailable()) {
            if let request = getUrlRequestForMultipart(config: config) {
                let task = session.uploadTask(with: request, from: request.httpBody){ (data, response, error) in
                    self.handleResponse(config: config, data: data, response: response, error: error, success: success, failure: failure)
                }
                task.resume()
            } else {
                failure(EBVHttpErrors.unableToCreateRequest)
            }
        } else {
            // send Internet is not available
            failure(EBVHttpErrors.noInternetConnection)
        }
    }
    
    fileprivate func getUrlRequestarray(config: HTTPRequestConfigArray) -> URLRequest? {
        
        var components = URLComponents()
        components.scheme = WebConstant.HttpScheme.secure
        components.host = config.baseUrl
        // Append path values
        if let path = config.path {
            components.path = path
        }
        
        //Add url params if provided
        if let urlParams = config.urlParams {
            var quaryArray = [URLQueryItem]()
            for urlParam in urlParams {
                quaryArray.append(URLQueryItem(name: urlParam.key, value: urlParam.value))
            }
            components.queryItems = quaryArray
        }
        guard let apiUrl = components.url else { return nil }
        var request = URLRequest(url: apiUrl)//apiUrl
        request.httpMethod = config.method.rawValue
        print("Request URL : %@",apiUrl)

        // Assign headers
        request.setValue(WebConstant.requestConstants.acceptType.json, forHTTPHeaderField: WebConstant.requestConstants.kContentType)
        request.setValue(WebConstant.requestConstants.acceptType.plainText, forHTTPHeaderField: WebConstant.requestConstants.kAcceptType)
        if let headers = config.headers {
            request.allHTTPHeaderFields = headers
        }
        
        //Add body params if provided
        if let bodyParams = config.body {
            if bodyParams.count > 0 {
                let jsonData = try? JSONSerialization.data(withJSONObject: bodyParams, options: [])
                request.httpBody = jsonData
                request.setValue("\(UInt(bodyParams.count))", forHTTPHeaderField: WebConstant.requestConstants.kContentLength)
            }
        }
        return request
    }

    
    fileprivate func getUrlRequest(config: HTTPRequestConfig) -> URLRequest? {
        
        var components = URLComponents()
        components.scheme = WebConstant.HttpScheme.secure
        components.host = config.baseUrl
        // Append path values
        if let path = config.path {
            components.path = path
        }
        
        //Add url params if provided
        if let urlParams = config.urlParams {
            var quaryArray = [URLQueryItem]()
            for urlParam in urlParams {
                quaryArray.append(URLQueryItem(name: urlParam.key, value: urlParam.value))
            }
            components.queryItems = quaryArray
        }
        guard let apiUrl = components.url else { return nil }
        var request = URLRequest(url: apiUrl)//apiUrl
        request.httpMethod = config.method.rawValue
        print("Request URL : %@",apiUrl)

        // Assign headers
        request.setValue(WebConstant.requestConstants.acceptType.json, forHTTPHeaderField: WebConstant.requestConstants.kContentType)
        request.setValue(WebConstant.requestConstants.acceptType.plainText, forHTTPHeaderField: WebConstant.requestConstants.kAcceptType)
        if let headers = config.headers {
            request.allHTTPHeaderFields = headers
        }
        
        //Add body params if provided
        if let bodyParams = config.body {
            if bodyParams.count > 0 {
                let jsonData = try? JSONSerialization.data(withJSONObject: bodyParams, options: [])
                request.httpBody = jsonData
                request.setValue("\(UInt(bodyParams.count))", forHTTPHeaderField: WebConstant.requestConstants.kContentLength)
            }
        }
        return request
    }
    
    
    fileprivate func getUrlRequestwithoutBody(config: HTTPRequestConfigwithoutBody) -> URLRequest? {
        
        var components = URLComponents()
        components.scheme = WebConstant.HttpScheme.secure
        components.host = config.baseUrl
        // Append path values
        if let path = config.path {
            components.path = path
        }
        
        //Add url params if provided
//        if let urlParams = config.urlParams {
//            var quaryArray = [URLQueryItem]()
//            for urlParam in urlParams {
//                quaryArray.append(URLQueryItem(name: urlParam.key, value: urlParam.value))
//            }
//            components.queryItems = quaryArray
//        }
        
        guard let url = URL(string: components.url!.absoluteString.removingPercentEncoding!) else { return nil }
//        guard let apiUrl = url else { return nil }
        var request = URLRequest(url: url)//apiUrl
        request.httpMethod = config.method.rawValue
        
        // Assign headers
        request.setValue(WebConstant.requestConstants.acceptType.json, forHTTPHeaderField: WebConstant.requestConstants.kContentType)
        request.setValue(WebConstant.requestConstants.acceptType.plainText, forHTTPHeaderField: WebConstant.requestConstants.kAcceptType)
        if let headers = config.headers {
            request.allHTTPHeaderFields = headers
        }
        print("Request URL : %@",url)
        //Add body params if provided
//        if let bodyParams = config.body {
//            if bodyParams.count > 0 {
//                let jsonData = try? JSONSerialization.data(withJSONObject: bodyParams, options: [])
//                request.httpBody = jsonData
//                request.setValue("\(UInt(bodyParams.count))", forHTTPHeaderField: WebConstant.requestConstants.kContentLength)
//            }
//        }
        return request
    }
    
     func getUrlRequestForMultipart(config: HTTPRequestConfig) -> URLRequest? {
        
        var components = URLComponents()
        components.scheme = WebConstant.HttpScheme.secure
        components.host = config.baseUrl
        // Append path values
        if let path = config.path {
            components.path = path
        }
        let httpBody = NSMutableData()
        let boundary = "Boundary-\(UUID().uuidString)"

        //Add url params if provided
        if let urlParams = config.urlParams {
            for (key, value) in urlParams {
                httpBody.appendString(convertFormField(named: key, value: value, using: boundary))
            }
        }
        guard let apiUrl = components.url else { return nil }
        var request = URLRequest(url: apiUrl)//apiUrl
        request.httpMethod = config.method.rawValue
        // Assign headers
      //  request.setValue(WebConstant.requestConstants.acceptType.multipart + "boundary=\(boundary)" , forHTTPHeaderField: WebConstant.requestConstants.kContentType)

        // Assign headers
        request.setValue(WebConstant.requestConstants.acceptType.json, forHTTPHeaderField: WebConstant.requestConstants.kContentType)
        request.setValue(WebConstant.requestConstants.acceptType.plainText, forHTTPHeaderField: WebConstant.requestConstants.kAcceptType)
        
        if let headers = config.headers {
            request.allHTTPHeaderFields = headers
        }
       
        //Add body params if provided
        if let bodyParams = config.body {
            if bodyParams.count > 0 {
                let jsonData = try? JSONSerialization.data(withJSONObject: bodyParams, options: [])
                request.httpBody = jsonData
                request.setValue("\(UInt(bodyParams.count))", forHTTPHeaderField: WebConstant.requestConstants.kContentLength)
            }
        }
        return request
    }

    
    fileprivate func handleResponse(config: HTTPRequestConfig, data:Data?,
                                    response:URLResponse?,
                                    error:Error?,
                                    success: @escaping (Data) -> (),
                                    failure: @escaping (Error) -> ()) {
        print("===========Responce=========\n%@",String(decoding: data ?? Data(), as: UTF8.self))
        if error != nil || data == nil {
            failure(error!)
        }
        guard let response1 = response as? HTTPURLResponse,
              (200...299).contains(response1.statusCode) else {
            // error from server
            let res = response as? HTTPURLResponse
            if res?.statusCode == 401{
                let accessToken = config.headers?["Authorization"]?.replacingOccurrences(of: "Bearer ", with: "")
                var refreshToken = ""
                var isUser = true
                if(accessToken == UserDefaultManager.shared[.accesstoken] as? String){
                    refreshToken = UserDefaultManager.shared[.refreshtoken] as? String ?? ""
                }
                else{
                    refreshToken = UserDefaultManager.shared[.rootrefreshtoken] as? String ?? ""
                    isUser = false
                }
                let param = [Constant.RequestConstant.refreshtoken:refreshToken] as [String : Any]
                AuthenticationManager.shared.getRefreshToken(paramers: param, isUser: isUser ,successBlock: { [self] (response) in
                    var config2 = config
                    if(isUser){
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)
                    }
                    else{
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)
                    }
                    makeRequest(config: config2, success: success, failure: failure)
                }, failureBlock: { error in
                    print("\n\n===========Error===========")
                    print("Error Code: \(error._code)")
                    print("Error Messsage: \(error.localizedDescription)")
                    Constant.appDelegate.GotoLogin()
                })
            }
            else{
                let decoder = JSONDecoder()
                let errorInfo = try? decoder.decode(ErrorInfo.self, from: data!)
                if errorInfo == nil{
                    convertError(failure: failure, statucode: res?.statusCode ?? 0)
                } else {
                    if (errorInfo?.error!.count)! > 0{
                        failure(EBVHttpErrors.customError((errorInfo?.error?[0].message)!))
                    } else {
                        failure(EBVHttpErrors.customError((errorInfo?.message)!))
                    }
                }

            }
             //convertError(failure: failure, error: error)
            //failure(ProcellHttpErrors.unacceptableStatusCode)
            return
        }
        
        guard let data = data else {
            failure(EBVHttpErrors.emptyData)
            return
        }
        
        success(data)
    }

    
    fileprivate func handleResponsearray(config: HTTPRequestConfigArray, data:Data?,
                                    response:URLResponse?,
                                    error:Error?,
                                    success: @escaping (Data) -> (),
                                    failure: @escaping (Error) -> ()) {
        print("===========Responce=========\n%@",String(decoding: data ?? Data(), as: UTF8.self))
        if error != nil || data == nil {
            failure(error!)
        }
        guard let response1 = response as? HTTPURLResponse,
              (200...299).contains(response1.statusCode) else {
            // error from server
            let res = response as? HTTPURLResponse
            if res?.statusCode == 401{
                let accessToken = config.headers?["Authorization"]?.replacingOccurrences(of: "Bearer ", with: "")
                var refreshToken = ""
                var isUser = true
                if(accessToken == UserDefaultManager.shared[.accesstoken] as? String){
                    refreshToken = UserDefaultManager.shared[.refreshtoken] as? String ?? ""
                }
                else{
                    refreshToken = UserDefaultManager.shared[.rootrefreshtoken] as? String ?? ""
                    isUser = false
                }
                let param = [Constant.RequestConstant.refreshtoken:refreshToken] as [String : Any]
                AuthenticationManager.shared.getRefreshToken(paramers: param, isUser: isUser ,successBlock: { [self] (response) in
                    var config2 = config
                    if(isUser){
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)
                    }
                    else{
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)
                    }
                    makeRequestarray(config: config2, success: success, failure: failure)
                }, failureBlock: { error in
                    print("\n\n===========Error===========")
                    print("Error Code: \(error._code)")
                    print("Error Messsage: \(error.localizedDescription)")
                    Constant.appDelegate.GotoLogin()
                    Constant.appDelegate.GotoLogin()
                })
                
            }
            else{
                let decoder = JSONDecoder()
                let errorInfo = try? decoder.decode(ErrorInfo.self, from: data!)
                if errorInfo == nil{
                    convertError(failure: failure, statucode: res?.statusCode ?? 0)
                } else {
                    if (errorInfo?.error!.count)! > 0{
                        failure(EBVHttpErrors.customError((errorInfo?.error?[0].message)!))
                    } else {
                        failure(EBVHttpErrors.customError((errorInfo?.message)!))
                    }
                }
            }
             //convertError(failure: failure, error: error)
            //failure(ProcellHttpErrors.unacceptableStatusCode)
            return
        }
        
        guard let data = data else {
            failure(EBVHttpErrors.emptyData)
            return
        }
        
        success(data)
    }

    
    fileprivate func handleResponsewithoutBody(config: HTTPRequestConfigwithoutBody, data:Data?,
                                    response:URLResponse?,
                                    error:Error?,
                                    success: @escaping (Data) -> (),
                                    failure: @escaping (Error) -> ()) {
        //print(String(decoding: data ?? Data(), as: UTF8.self))
        if error != nil || data == nil {
            failure(error!)
        }
        guard let response1 = response as? HTTPURLResponse,
              (200...299).contains(response1.statusCode) else {
            // error from server
            let res = response as? HTTPURLResponse
            if res?.statusCode == 401{
                let accessToken = config.headers?["Authorization"]?.replacingOccurrences(of: "Bearer ", with: "")
                var refreshToken = ""
                var isUser = true
                if(accessToken == UserDefaultManager.shared[.accesstoken] as? String){
                    refreshToken = UserDefaultManager.shared[.refreshtoken] as? String ?? ""
                }
                else{
                    refreshToken = UserDefaultManager.shared[.rootrefreshtoken] as? String ?? ""
                    isUser = false
                }
                let param = [Constant.RequestConstant.refreshtoken:refreshToken] as [String : Any]
                AuthenticationManager.shared.getRefreshToken(paramers: param, isUser: isUser ,successBlock: { [self] (response) in
                    var config2 = config
                    if(isUser){
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)
                    }
                    else{
                        config2.headers?["Authorization"] = "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)
                    }
                    makeRequestwithoutBody(config: config2, success: success, failure: failure)
                }, failureBlock: { error in
                    print("\n\n===========Error===========")
                    print("Error Code: \(error._code)")
                    print("Error Messsage: \(error.localizedDescription)")
                    Constant.appDelegate.GotoLogin()
                })
            }
            else{
                convertError(failure: failure, statucode: res?.statusCode ?? 0)
            }
             //convertError(failure: failure, error: error)
            //failure(ProcellHttpErrors.unacceptableStatusCode)
            return
        }
        
        guard let data = data else {
            failure(EBVHttpErrors.emptyData)
            return
        }
        success(data)
    }
    
    
    

    
    fileprivate func convertError(failure: @escaping (Error) -> (),statucode:Int ){
//        guard let error = error  else {
//            failure(ProcellHttpErrors.unknownError)
//            return
//        }
     
        switch statucode {
        case 404:
            failure(EBVHttpErrors.notFound)
        case 401:
            failure(EBVHttpErrors.unAuthorized)
        case 403:
            failure(EBVHttpErrors.forbidden)
        default:
            failure(EBVHttpErrors.unknownError)
        }
    }
    
    fileprivate func convertError(failure: @escaping (Error) -> (),error:Error? ){
        guard let error = error  else {
            failure(EBVHttpErrors.unknownError)
            return
        }
     
        switch error.code {
        case 404:
            failure(EBVHttpErrors.notFound)
        case 401:
            failure(EBVHttpErrors.unAuthorized)
        case 403:
            failure(EBVHttpErrors.forbidden)
        default:
            failure(EBVHttpErrors.unknownError)
        }
    }
    
    fileprivate func checkInternetAvailable() -> Bool {
        let networkStatus = try! Reachability().connection
        switch networkStatus {
        case nil:
            return false
        case .cellular:
            return true
        case .wifi:
            return true
        case .none, .unavailable:
            return false
        }
    }
    
    func convertFormField(named name: String, value: Any, using boundary: String) -> String {
      var fieldString = "--\(boundary)\r\n"
      fieldString += "Content-Disposition: form-data; name=\"\(name)\"\r\n"
      fieldString += "\r\n"
      fieldString += "\(value)\r\n"
      return fieldString
    }
    
    func convertFileData(fieldName: String, fileName: String, mimeType: String, fileData: Data, using boundary: String) -> Data {
      let data = NSMutableData()
      data.appendString("--\(boundary)\r\n")
      data.appendString("Content-Disposition: form-data; name=\"\(fieldName)\"; filename=\"\(fileName)\"\r\n")
      data.appendString("Content-Type: \(mimeType)\r\n\r\n")
      data.append(fileData)
      data.appendString("\r\n")

      return data as Data
    }
}




