/*
 * drum_oled.c
 *
 *  Created on: 15. feb. 2022
 *      Author: MaliU
 */

#include "ssd1327.h"

#include "drum_oled.h"
#include "drum_fonts.h"

static uint8_t OLED_Buffer[OLED_BUFFERSIZE];

OLED_t OLED = {
               0,
               0,
               0,
               0,
               0,
               OLED_WIDTH - 1,
               0,
               OLED_HEIGHT - 1,
               OLED_Buffer
};

void OLED_Init(){

#if OLED_TYPE == OLED_TYPE_SSD1327
    SSD1327_Init();
#endif

    /* Clear screen */
    OLED_Fill(OLED_COLOR_BLACK);

    /* Set default values */
    OLED.CurrentX = 0;
    OLED.CurrentY = 0;

    /* Update screen */
    OLED_UpdateScreen();

    /* Initialized OK */
    OLED.Initialized = 1;
}

void OLED_SetContrast(uint8_t contrast){

#if OLED_TYPE == OLED_TYPE_SSD1327
    SSD1327_SetContrast(contrast);
#endif
}

void OLED_UpdateScreen(){
#if OLED_TYPE == OLED_TYPE_SSD1327
    SSD1327_UpdateScreen(OLED);
#endif
}


void OLED_Fill(OLED_COLOR_t color) {
    uint8_t pixel_colors;
    if(color > OLED_COLOR_WHITE) color = OLED_COLOR_WHITE;

#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_2)
    pixel_colors = (color == OLED_COLOR_WHITE) ? 0xFF : 0x00;
#endif
#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
    pixel_colors = color << 6 | color << 4 | color << 2 | color;
#endif
#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_16)
    pixel_colors = color << 4 | color;
#endif

    /* Set memory */
    memset(OLED.Buffer, (pixel_colors), OLED_BUFFERSIZE);

    OLED.MinUpdatedX = 0;
    OLED.MaxUpdatedX = OLED_WIDTH - 1;
    OLED.MinUpdatedY = 0;
    OLED.MaxUpdatedY = OLED_HEIGHT - 1;
}

void OLED_DrawPixel(uint16_t x, uint16_t y, OLED_COLOR_t color) {
    uint16_t temp;
    if(color > OLED_COLOR_WHITE) color = OLED_COLOR_WHITE;

#if (OLED_ROTATE == OLED_ROTATE_90)
    //rotate 90
    temp = x;
    x = y;
    y = OLED_WIDTH-1 - temp;
#endif

#if (OLED_ROTATE == OLED_ROTATE_180)
    // rotate 180

    x = OLED_WIDTH-1 - x;
    y = OLED_HEIGHT-1 - y;
#endif

#if (OLED_ROTATE == OLED_ROTATE_270)
    //rotate 270
    temp = y;
    y = x;
    x = OLED_WIDTH-1 - temp;
#endif


    //if ((x < 0) || (x >= OLED_WIDTH) || (y < 0) || (y >= OLED_HEIGHT))
    if ((x >= OLED_WIDTH) || (y >= OLED_HEIGHT)) //x and y are unsigned ints
        return;



    //uint16_t pixel_buffer_location = x/2 + y*(OLED_WIDTH/2);
    uint16_t pixel_buffer_location = (x + y*OLED_WIDTH) / (8 / OLED_GRAYSCALE_LEVELS);
    uint8_t pixel = OLED.Buffer[pixel_buffer_location];

#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_2)
    pixel_colors = (color == OLED_COLOR_WHITE) ? 0xFF : 0x00;
#endif
#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
    //horizontal addressing (SSD1327) MSB bits are left pixel
    switch(x % 4){
        case 0: //most left pixel
            pixel &= ~(0xC0);
            pixel |= (0xC0 & (color << 6));
            break;
        case 1: //
            pixel &= ~(0x30);
            pixel |= (0x30 & (color << 4));
            break;
        case 2: //
            pixel &= ~(0x0C);
            pixel |= (0x0C & (color << 2));
            break;
        case 3: //most right pixel
            pixel &= ~(0x03);
            pixel |= (0x03 & (color));
            break;
    }
#endif

#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_16)
    //horizontal addressing (SSD1327) MSB bits are left pixel
    if(x % 2)
    {
        pixel &= ~(0x0F);
        pixel |= (0x0F & color);
    }
    else
    {
        pixel &= ~(0xF0);
        pixel |= (0xF0 & (color<<4));
    }
#endif

    if( OLED.Buffer[pixel_buffer_location] != pixel){
        //pixel was updated
        if(x > OLED.MaxUpdatedX) OLED.MaxUpdatedX = x;
        if(x < OLED.MinUpdatedX) OLED.MinUpdatedX = x;
        if(y > OLED.MaxUpdatedY) OLED.MaxUpdatedY = y;
        if(y < OLED.MinUpdatedY) OLED.MinUpdatedY = y;
    }

    OLED.Buffer[pixel_buffer_location] = pixel;
}


void OLED_GotoXY(uint16_t x, uint16_t y) {
  /* Set write pointers */
  OLED.CurrentX = x;
  OLED.CurrentY = y;
}

unsigned char OLED_Putc(unsigned char ch, DrUM_FontDef_t* Font, OLED_COLOR_t text_color, OLED_COLOR_t back_color, uint8_t enable_background) {
    uint32_t i, b, j;
    uint8_t chWidth, chHeight;
    uint32_t line;
    uint32_t chStart;
    //uint32_t shift =  1 << Font->FontHeight;

    switch(Font->FontType){
        case FONT_MIKROE_GLDC_1V_VAR : //this type of fonts draws columns, LSB is top pixel, MSB is bottom pixel, from left to right
            if(ch > Font->FontLastChar) ch = Font->FontLastChar;
            chStart = (ch - Font->FontFirstChar) * (Font->FontWidth + 1);
            chWidth = Font->data[chStart++];
            chHeight = Font->FontHeight;

            for(i = 0; i < chWidth; i++){
                line = Font->data[chStart++];
                for (j = 0; j < Font->FontHeight; j++){
                    if(line & 0x0001){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, text_color);
                    } else {
                        if(enable_background){ //enable background. we can overlay different colors
                            OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                        }
                    }
                    line >>= 1; //shift right for one pixel
                }
            }
            OLED.CurrentX += chWidth;
            //add char spacing
            if(enable_background){ //enable background. we can overlay different colors
                for(i = 0; i < Font->FontSpaceH; i++){
                    line = 0;
                    for (j = 0; j < Font->FontHeight; j++){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                    }
                }
            }
            OLED.CurrentX += Font->FontSpaceH;
            break;

        case FONT_MIKROE_GLDC_2V_VAR : //this type of fonts draws columns, LSB is top pixel, MSB is bottom pixel, from left to right
            if(ch > Font->FontLastChar) ch = Font->FontLastChar;
            chStart = (ch - Font->FontFirstChar) * (2*Font->FontWidth + 1);
            chWidth = Font->data[chStart++];
            chHeight = Font->FontHeight;

            for(i = 0; i < chWidth; i++){
                line = Font->data[chStart++] | Font->data[chStart++] << 8;
                for (j = 0; j < Font->FontHeight; j++){
                    if(line & 0x0001){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, text_color);
                    } else {
                        if(enable_background){ //enable background. we can overlay different colors
                            OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                        }
                    }
                    line >>= 1; //shift right for one pixel
                }
            }
            OLED.CurrentX += chWidth;
            //add char spacing
            if(enable_background){ //enable background. we can overlay different colors
                for(i = 0; i < Font->FontSpaceH; i++){
                    line = 0;
                    for (j = 0; j < Font->FontHeight; j++){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                    }
                }
            }
            OLED.CurrentX += Font->FontSpaceH;
            break;
        case FONT_MIKROE_GLDC_4V_VAR : //this type of fonts draws columns, LSB is top pixel, MSB is bottom pixel, from left to right
            if(ch > Font->FontLastChar) ch = Font->FontLastChar;
            chStart = (ch - Font->FontFirstChar) * (4*Font->FontWidth + 1);
            chWidth = Font->data[chStart++];
            chHeight = Font->FontHeight;

            for(i = 0; i < chWidth; i++){
                line = Font->data[chStart++] | Font->data[chStart++] << 8 | Font->data[chStart++] << 16 | Font->data[chStart++] << 24;
                for (j = 0; j < Font->FontHeight; j++){
                    if(line & 0x0001){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, text_color);
                    } else {
                        if(enable_background){ //enable background. we can overlay different colors
                            OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                        }
                    }
                    line >>= 1; //shift right for one pixel
                }
            }
            OLED.CurrentX += chWidth;
            //add char spacing
            if(enable_background){ //enable background. we can overlay different colors
                for(i = 0; i < Font->FontSpaceH; i++){
                    line = 0;
                    for (j = 0; j < Font->FontHeight; j++){
                        OLED_DrawPixel(OLED.CurrentX + i, OLED.CurrentY + j, back_color);
                    }
                }
            }
            OLED.CurrentX += Font->FontSpaceH;
            break;
        }

    /* Return character written */
    return ch;
}


unsigned char OLED_PrintXY(uint16_t x, uint16_t y, DrUM_FontDef_t* Font, OLED_COLOR_t text_color, OLED_COLOR_t back_color, uint8_t enable_background, const char* format, ...) {

    //xSemaphoreTake(mutex_frame_buffer, portMAX_DELAY);

    /* Set write pointers */
    OLED.CurrentX = x;
    OLED.CurrentY = y;

    va_list args;
    va_start(args, format);
    const unsigned char* str;
    unsigned char tmp[64];

    vsnprintf(tmp, sizeof(tmp), format, args);

    str = tmp;

    /* Write characters */
    while (*str) {
    /* Write character by character */
    if (OLED_Putc(*str, Font, text_color, back_color, enable_background) != *str) {
        /* Return error */
        return *str;
    }

        /* Increase string pointer */
        str++;
    }

    //xSemaphoreGive(mutex_frame_buffer);

    /* Everything OK, zero should be returned */
    return *str;
}

//void SSD1327_DrawLine(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, SSD1327_COLOR_t c) {
//  int16_t dx, dy, sx, sy, err, e2, i, tmp;
//
//  /* Check for overflow */
//  if (x0 >= SSD1327_WIDTH) {
//    x0 = SSD1327_WIDTH - 1;
//  }
//  if (x1 >= SSD1327_WIDTH) {
//    x1 = SSD1327_WIDTH - 1;
//  }
//  if (y0 >= SSD1327_HEIGHT) {
//    y0 = SSD1327_HEIGHT - 1;
//  }
//  if (y1 >= SSD1327_HEIGHT) {
//    y1 = SSD1327_HEIGHT - 1;
//  }
//
//  dx = (x0 < x1) ? (x1 - x0) : (x0 - x1);
//  dy = (y0 < y1) ? (y1 - y0) : (y0 - y1);
//  sx = (x0 < x1) ? 1 : -1;
//  sy = (y0 < y1) ? 1 : -1;
//  err = ((dx > dy) ? dx : -dy) / 2;
//
//  if (dx == 0) {
//    if (y1 < y0) {
//      tmp = y1;
//      y1 = y0;
//      y0 = tmp;
//    }
//
//    if (x1 < x0) {
//      tmp = x1;
//      x1 = x0;
//      x0 = tmp;
//    }
//
//    /* Vertical line */
//    for (i = y0; i <= y1; i++) {
//      SSD1327_DrawPixel(x0, i, c);
//    }
//
//    /* Return from function */
//    return;
//  }
//
//  if (dy == 0) {
//    if (y1 < y0) {
//      tmp = y1;
//      y1 = y0;
//      y0 = tmp;
//    }
//
//    if (x1 < x0) {
//      tmp = x1;
//      x1 = x0;
//      x0 = tmp;
//    }
//
//    /* Horizontal line */
//    for (i = x0; i <= x1; i++) {
//      SSD1327_DrawPixel(i, y0, c);
//    }
//
//    /* Return from function */
//    return;
//  }
//
//  while (1) {
//    SSD1327_DrawPixel(x0, y0, c);
//    if (x0 == x1 && y0 == y1) {
//      break;
//    }
//    e2 = err;
//    if (e2 > -dx) {
//      err -= dy;
//      x0 += sx;
//    }
//    if (e2 < dy) {
//      err += dx;
//      y0 += sy;
//    }
//  }
//}

//void SSD1327_DrawRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1327_COLOR_t c) {
//  /* Check input parameters */
//  if (
//      x >= SSD1327_WIDTH ||
//        y >= SSD1327_HEIGHT
//          ) {
//            /* Return error */
//            return;
//          }
//
//  /* Check width and height */
//  if ((x + w) >= SSD1327_WIDTH) {
//    w = SSD1327_WIDTH - x;
//  }
//  if ((y + h) >= SSD1327_HEIGHT) {
//    h = SSD1327_HEIGHT - y;
//  }
//
//  /* Draw 4 lines */
//  SSD1327_DrawLine(x, y, x + w, y, c);         /* Top line */
//  SSD1327_DrawLine(x, y + h, x + w, y + h, c); /* Bottom line */
//  SSD1327_DrawLine(x, y, x, y + h, c);         /* Left line */
//  SSD1327_DrawLine(x + w, y, x + w, y + h, c); /* Right line */
//}
//
//void SSD1327_DrawFilledRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1327_COLOR_t c) {
//  uint8_t i;
//
//  /* Check input parameters */
//  if (
//      x >= SSD1327_WIDTH ||
//        y >= SSD1327_HEIGHT
//          ) {
//            /* Return error */
//            return;
//          }
//
//  /* Check width and height */
//  if ((x + w) >= SSD1327_WIDTH) {
//    w = SSD1327_WIDTH - x;
//  }
//  if ((y + h) >= SSD1327_HEIGHT) {
//    h = SSD1327_HEIGHT - y;
//  }
//
//  /* Draw lines */
//  for (i = 0; i <= h; i++) {
//    /* Draw lines */
//    SSD1327_DrawLine(x, y + i, x + w, y + i, c);
//  }
//}
//
//void SSD1327_DrawTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1327_COLOR_t color) {
//  /* Draw lines */
//  SSD1327_DrawLine(x1, y1, x2, y2, color);
//  SSD1327_DrawLine(x2, y2, x3, y3, color);
//  SSD1327_DrawLine(x3, y3, x1, y1, color);
//}
//
//void SSD1327_DrawFilledTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1327_COLOR_t color) {
//  int16_t deltax = 0, deltay = 0, x = 0, y = 0, xinc1 = 0, xinc2 = 0,
//  yinc1 = 0, yinc2 = 0, den = 0, num = 0, numadd = 0, numpixels = 0,
//  curpixel = 0;
//
//  deltax = ABS(x2 - x1);
//  deltay = ABS(y2 - y1);
//  x = x1;
//  y = y1;
//
//  if (x2 >= x1) {
//    xinc1 = 1;
//    xinc2 = 1;
//  } else {
//    xinc1 = -1;
//    xinc2 = -1;
//  }
//
//  if (y2 >= y1) {
//    yinc1 = 1;
//    yinc2 = 1;
//  } else {
//    yinc1 = -1;
//    yinc2 = -1;
//  }
//
//  if (deltax >= deltay){
//    xinc1 = 0;
//    yinc2 = 0;
//    den = deltax;
//    num = deltax / 2;
//    numadd = deltay;
//    numpixels = deltax;
//  } else {
//    xinc2 = 0;
//    yinc1 = 0;
//    den = deltay;
//    num = deltay / 2;
//    numadd = deltax;
//    numpixels = deltay;
//  }
//
//  for (curpixel = 0; curpixel <= numpixels; curpixel++) {
//    SSD1327_DrawLine(x, y, x3, y3, color);
//
//    num += numadd;
//    if (num >= den) {
//      num -= den;
//      x += xinc1;
//      y += yinc1;
//    }
//    x += xinc2;
//    y += yinc2;
//  }
//}
//
//void SSD1327_DrawCircle(int16_t x0, int16_t y0, int16_t r, SSD1327_COLOR_t c) {
//  int16_t f = 1 - r;
//  int16_t ddF_x = 1;
//  int16_t ddF_y = -2 * r;
//  int16_t x = 0;
//  int16_t y = r;
//
//  SSD1327_DrawPixel(x0, y0 + r, c);
//  SSD1327_DrawPixel(x0, y0 - r, c);
//  SSD1327_DrawPixel(x0 + r, y0, c);
//  SSD1327_DrawPixel(x0 - r, y0, c);
//
//  while (x < y) {
//    if (f >= 0) {
//      y--;
//      ddF_y += 2;
//      f += ddF_y;
//    }
//    x++;
//    ddF_x += 2;
//    f += ddF_x;
//
//    SSD1327_DrawPixel(x0 + x, y0 + y, c);
//    SSD1327_DrawPixel(x0 - x, y0 + y, c);
//    SSD1327_DrawPixel(x0 + x, y0 - y, c);
//    SSD1327_DrawPixel(x0 - x, y0 - y, c);
//
//    SSD1327_DrawPixel(x0 + y, y0 + x, c);
//    SSD1327_DrawPixel(x0 - y, y0 + x, c);
//    SSD1327_DrawPixel(x0 + y, y0 - x, c);
//    SSD1327_DrawPixel(x0 - y, y0 - x, c);
//  }
//}
//
//void SSD1327_DrawFilledCircle(int16_t x0, int16_t y0, int16_t r, SSD1327_COLOR_t c) {
//  int16_t f = 1 - r;
//  int16_t ddF_x = 1;
//  int16_t ddF_y = -2 * r;
//  int16_t x = 0;
//  int16_t y = r;
//
//  SSD1327_DrawPixel(x0, y0 + r, c);
//  SSD1327_DrawPixel(x0, y0 - r, c);
//  SSD1327_DrawPixel(x0 + r, y0, c);
//  SSD1327_DrawPixel(x0 - r, y0, c);
//  SSD1327_DrawLine(x0 - r, y0, x0 + r, y0, c);
//
//  while (x < y) {
//    if (f >= 0) {
//      y--;
//      ddF_y += 2;
//      f += ddF_y;
//    }
//    x++;
//    ddF_x += 2;
//    f += ddF_x;
//
//    SSD1327_DrawLine(x0 - x, y0 + y, x0 + x, y0 + y, c);
//    SSD1327_DrawLine(x0 + x, y0 - y, x0 - x, y0 - y, c);
//
//    SSD1327_DrawLine(x0 + y, y0 + x, x0 - y, y0 + x, c);
//    SSD1327_DrawLine(x0 + y, y0 - x, x0 - y, y0 - x, c);
//  }
//}
