/******************************************************************************
 * File Name:   main.c
 *
 * Description: This is the source code for MQTT Client Example for ModusToolbox.
 *
 * Related Document: See README.md
 *
 *
 *******************************************************************************
 * Copyright 2020-2021, Cypress Semiconductor Corporation (an Infineon company) or
 * an affiliate of Cypress Semiconductor Corporation.  All rights reserved.
 *
 * This software, including source code, documentation and related
 * materials ("Software") is owned by Cypress Semiconductor Corporation
 * or one of its affiliates ("Cypress") and is protected by and subject to
 * worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions.
 * Therefore, you may use this Software only as provided in the license
 * agreement accompanying the software package from which you
 * obtained this Software ("EULA").
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
 * non-transferable license to copy, modify, and compile the Software
 * source code solely for use in connection with Cypress's
 * integrated circuit products.  Any reproduction, modification, translation,
 * compilation, or representation of this Software except as specified
 * above is prohibited without the express written permission of Cypress.
 *
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
 * reserves the right to make changes to the Software without notice. Cypress
 * does not assume any liability arising out of the application or use of the
 * Software or any product or circuit described in the Software. Cypress does
 * not authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death ("High Risk Product"). By
 * including Cypress's product in a High Risk Product, the manufacturer
 * of such system or application assumes all risk of such use and in doing
 * so agrees to indemnify Cypress against all liability.
 *******************************************************************************/


/* Header file includes */
#include "cyhal.h"
#include "cybsp.h"
#include "cy_retarget_io.h"
#include "cy_rgb_led.h"
#include "cy_log.h"

#include "FreeRTOS.h"
#include "task.h"

#include "optiga_trust_task.h"
#include "optiga/pal/pal_i2c.h"

#include "drum_oled.h"
#include "drum_fonts.h"

#include "ebv_logs.h"

#include "app_ver.h"


/******************************************************************************
 * Global Variables
 ******************************************************************************/
/* This enables RTOS aware debugging. */
volatile int uxTopUsedPriority;




/******************************************************************************
 * Function Name: main
 ******************************************************************************
 * Summary:
 *  System entrance point. This function initializes retarget IO, sets up 
 *  the MQTT client task, and then starts the RTOS scheduler.
 *
 * Parameters:
 *  void
 *
 * Return:
 *  int
 *
 ******************************************************************************/
int main() {
    cy_rslt_t result;
#ifdef OTA_SUPPORT
    cyhal_wdt_t wdt_obj;
#endif

    /* This enables RTOS aware debugging in OpenOCD.  6- max 0-min priority*/
    uxTopUsedPriority = configMAX_PRIORITIES - 1;

    /* Initialize the board support package. */
    result = cybsp_init();
    CY_ASSERT(CY_RSLT_SUCCESS == result);

    /* To avoid compiler warnings. */
    (void) result;

    /* Enable global interrupts. */
    __enable_irq();

#ifdef ENABLE_SECURE_SOCKETS_LOGS
    /*
        This function is required only if there are some problems with the
        secure sockets (pkcs11 interface to the optiga chip) and you would
        like to understand better what is happening. Additionally add the
        following Macros for the Makefile list of DEFINES
        CY_SECURE_SOCKETS_PKCS_SUPPORT MBEDTLS_VERBOSE=4
    */
    cy_rslt_t rs = cy_log_init(CY_LOG_MAX, app_log_output_callback, app_log_time);
    if (rs != CY_RSLT_SUCCESS)
    {
        printf("cy_log_init() FAILED %ld\n", result);
    }
#endif

    /* default for all logging to WARNING */
    cy_log_init(CY_LOG_WARNING, NULL, NULL);

    /* Initialize retarget-io to use the debug UART port. */
	result = cy_retarget_io_init(CYBSP_DEBUG_UART_TX, CYBSP_DEBUG_UART_RX,
	CY_RETARGET_IO_BAUDRATE);
	EBV_LOGS_PRINTF("======================================================================\n");
	EBV_LOGS_PRINTF("EBV-IoT - Infineon Xensiv & Avnet IoTConnect Demo v%s \n", APP_VERSION);
	EBV_LOGS_PRINTF("======================================================================\n");

	if (result != CY_RSLT_SUCCESS)
	{
		CY_ASSERT(0);
	}

#ifdef OTA_SUPPORT
    /* Clear watchdog timer so that it doesn't trigger a reset */
    cyhal_wdt_init(&wdt_obj, cyhal_wdt_get_max_timeout_ms());
    cyhal_wdt_free(&wdt_obj);
    printf("\nWatchdog timer started by the bootloader is now turned off!!!\n\n");
#endif

	/* Init RGB LED */
    result = cy_rgb_led_init(CYBSP_LED_RGB_RED, CYBSP_LED_RGB_GREEN, CYBSP_LED_RGB_BLUE, CY_RGB_LED_ACTIVE_LOW);
    /* GPIO init failed. Stop program execution */
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }
    cy_rgb_led_on(CY_RGB_LED_COLOR_WHITE, 0); //set the color to white, no brightness
    cy_rgb_led_off(); //turn off led

    /* Create an OPTIGA task to make sure everything related to
     * the OPTIGA stack will be called from the scheduler */
    xTaskCreate(optiga_client_task, "Optiga Client Task", OPTIGA_TRUST_TASK_STACK_SIZE, NULL, OPTIGA_TRUST_TASK_PRIORITY, NULL);
    /* Start the FreeRTOS scheduler. */
    vTaskStartScheduler();

    /* Should never get here. */
    CY_ASSERT(0);
}

/* [] END OF FILE */
