/*
 * sensor_task.c
 *
 *  Created on: 06.04.2023.
 *      Author: ilicb
 */

/* FreeRTOS header files */
#include "FreeRTOS.h"
#include "task.h"

#include "cyhal.h"
#include "cybsp.h"

#include "drum_oled.h"
#include "drum_fonts.h"

#include "xensiv_pasco2_mtb.h"
#include "xensiv_dps3xx_mtb.h"

#include "sensor_task.h"

#include "xensiv_pasco2_mtb.h"
#include "xensiv_dps3xx_mtb.h"

#include "drum_oled.h"
#include "drum_fonts.h"

#include "cy_rgb_led.h"
#include "ebv_logs.h"


#if defined(TARGET_CYSBSYSKIT_DEV_01)
/* Output pin for sensor PSEL line */
#define MTB_PASCO2_PSEL (P5_3)
/* Output pin for PAS CO2 Wing Board power switch */
#define MTB_PASCO2_POWER_SWITCH (P10_5)
/* Output pin for PAS CO2 Wing Board LED OK */
#define MTB_PASCO2_LED_OK (P9_0)
/* Output pin for PAS CO2 Wing Board LED WARNING  */
#define MTB_PASCO2_LED_WARNING (P9_1)
#endif

/* Pin state to enable I2C channel of sensor */
#define MTB_PASCO2_PSEL_I2C_ENABLE (0U)
/* Pin state to enable power to sensor on PAS CO2 Wing Board*/
#define MTB_PASCO2_POWER_ON (1U)
/* Pin state for PAS CO2 Wing Board LED off. */
#define MTB_PASCO_LED_STATE_OFF (0U)
/* Pin state for PAS CO2 Wing Board LED on. */
#define MTB_PASCO_LED_STATE_ON (1U)

#define WAIT_SENSOR_RDY_MS          (2000)      /* Wait time for sensor ready (milliseconds) */

/* Delay time after each PAS CO2 readout */
#define PASCO2_PROCESS_DELAY (1000)

#define SENSOR_TASK_INTERVAL_MS 	(100)
#define DPS_SENSOR_UPDATE_COUNT 	(20) //cca 2s
#define PAS_CO2_SENSOR_UPDATE_COUNT 	(50) //cca 5s
#define DEFAULT_PRESSURE_REF_HPA    (0x3F7)     /* Default atmospheric pressure to compensate for (hPa) */

#define STRING_BUFFER_SIZE (80)

//extern xensiv_pasco2_t xensiv_pasco2;
//extern xensiv_dps3xx_t pressure_sensor;

xensiv_pasco2_t xensiv_pasco2;
xensiv_dps3xx_t pressure_sensor;

uint8_t xensiv_pasco2_available = false, pressure_sensor_avaialble = false;

extern cyhal_i2c_t i2c_master_obj;

uint16_t ppm = 0;
uint16_t sensor_count = 0;
float pressure, temperature;

cy_rslt_t result;
cyhal_rtc_t rtc_obj;
uint8_t lastSecond = 0;

static const char *TAG = "sensor_task";

static void pasco2_init(void)
{
    /* Initialize the User LED */
    result = cyhal_gpio_init(CYBSP_USER_LED, CYHAL_GPIO_DIR_OUTPUT,
                             CYHAL_GPIO_DRIVE_STRONG, CYBSP_LED_STATE_OFF);
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }
	//vTaskDelay(pdMS_TO_TICKS(WAIT_SENSOR_RDY_MS));
    // Initialize PAS CO2 sensor with default parameter values
    result = xensiv_pasco2_mtb_init_i2c(&xensiv_pasco2, &i2c_master_obj);
    if (result != CY_RSLT_SUCCESS)
    {
        //printf("PAS CO2 device initialization error\n");
        EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "PAS CO2 device initialization error - sensor not available");
        //printf("Exiting pasco2_task task\n");
        // exit current thread (suspend)
        xensiv_pasco2_available = false;
    } else {
    	xensiv_pasco2_available = true;
    }

    // Configure PAS CO2 Wing board interrupt to enable 12V boost converter in wingboard
    xensiv_pasco2_interrupt_config_t int_config =
    {
        .b.int_func = XENSIV_PASCO2_INTERRUPT_FUNCTION_NONE,
        .b.int_typ = (uint32_t)XENSIV_PASCO2_INTERRUPT_TYPE_LOW_ACTIVE
    };

    if (xensiv_pasco2_available == true){
		result = xensiv_pasco2_set_interrupt_config(&xensiv_pasco2, int_config);
		if (result != CY_RSLT_SUCCESS)
		{
			EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "PAS CO2 interrupt configuration error - sensor not available");
			xensiv_pasco2_available = false;
		} else {
			xensiv_pasco2_available = true;
			//    result = xensiv_pasco2_set_measurement_rate(&xensiv_pasco2, XENSIV_PASCO2_MEAS_RATE_MIN);
			//	if (result != CY_RSLT_SUCCESS)
			//	{
			//		printf("PAS CO2 measurement rate configuration error");
			//		CY_ASSERT(0);
			//	}


			cyhal_gpio_write(CYBSP_USER_LED, false); // USER_LED is active low

			// Turn on status LED on PAS CO2 Wing Board to indicate normal operation
			// cyhal_gpio_write(MTB_PASCO2_LED_OK, MTB_PASCO_LED_STATE_ON);

			EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "PAS CO2 initialized successfully");

		}
    }

    /* Initialize pressure sensor */
	result = xensiv_dps3xx_mtb_init_i2c(&pressure_sensor, &i2c_master_obj, XENSIV_DPS3XX_I2C_ADDR_DEFAULT);
	if (result != CY_RSLT_SUCCESS)
	{
		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "DPS device initialization error - sensor not available");
		pressure_sensor_avaialble = false;
	} else {
		pressure_sensor_avaialble = true;
		EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Pressure sensor initialized successfully");
	}
}



void sensor_task(void *pvParameters) {
	struct tm date_time;
	//char buffer[STRING_BUFFER_SIZE];

	OLED_Init();

	OLED_PrintXY(10, 0, &IsidoraSansMedium16x16, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "%cEBV-IoT%c ", 127, 127);

	OLED_UpdateScreen();

	/* Initialize PAS CO2 sensor */
    pasco2_init();

	if(xensiv_pasco2_available == true){
		OLED_PrintXY(2, 40, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Infineon PAS CO2");
	} else {
		if (pressure_sensor_avaialble == true){
			OLED_PrintXY(2, 40, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Infineon DPS310 ");
		} else {
			OLED_PrintXY(2, 40, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, " EBV & Infineon ");
		}
	}
	OLED_UpdateScreen();

    /* Initialize RTC */
	result = cyhal_rtc_init(&rtc_obj);
	if (CY_RSLT_SUCCESS != result)
	{
		//printf("RTC initialization failed!\n\n");
		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "RTC initialization failed!");
	}

	result = cyhal_rtc_read(&rtc_obj, &date_time);
	if (CY_RSLT_SUCCESS == result)
	{
		//update time
		if(lastSecond != date_time.tm_sec){ //Update seconds
			//OLED_PrintXY(2, 23, &MS_Gothic6x12, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
			OLED_PrintXY(2, 23, &Small5x8, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
					"%02d/%02d/%02d,%02d:%02d:%02d",
					date_time.tm_year%100, date_time.tm_mon+1, date_time.tm_mday,
					date_time.tm_hour, date_time.tm_min, date_time.tm_sec);
			OLED_UpdateScreen();
		}
		lastSecond = date_time.tm_sec;
	}

    //sensor are read every second. PAS CO2 values is eventually only updated every second
	while(1)
	{
		//read new time
		result = cyhal_rtc_read(&rtc_obj, &date_time);
		if (CY_RSLT_SUCCESS == result)
		{
			//update time
			if(lastSecond != date_time.tm_sec){ //Update seconds
				//OLED_PrintXY(2, 23, &MS_Gothic6x12, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
				OLED_PrintXY(2, 23, &Small5x8, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
						"%02d/%02d/%02d,%02d:%02d:%02d",
						date_time.tm_year%100, date_time.tm_mon+1, date_time.tm_mday,
						date_time.tm_hour, date_time.tm_min, date_time.tm_sec);
				OLED_UpdateScreen();
			}
			lastSecond = date_time.tm_sec;
		}

	    if ((sensor_count % DPS_SENSOR_UPDATE_COUNT) == 0){

	    	/* Read CO2 value from sensor */
			//float32_t pressure = DEFAULT_PRESSURE_VALUE;
	    	if(xensiv_pasco2_available == true){
				result = xensiv_pasco2_mtb_read(&xensiv_pasco2, (uint16_t)pressure, &ppm); //unit PPM
				do{
					xensiv_pasco2_mtb_read(&xensiv_pasco2,DEFAULT_PRESSURE_REF_HPA, &ppm); //unit PPM
					}
				while(ppm == 0);
	    	}
	//		if (result == CY_RSLT_SUCCESS)
	//		{
	//			printf("READ SUCCESS\r\n");
	//		}

			if (pressure_sensor_avaialble == true){
				result = xensiv_dps3xx_read(&pressure_sensor, &pressure, &temperature);
			}

			if (xensiv_pasco2_available == true){
				OLED_PrintXY(25, 58, &ArialRoundedMTBold28x31, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
													"%d   ",
													ppm);

				OLED_PrintXY(46, 90, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
															"ppm ");
			} else {
				OLED_PrintXY(25, 58, &ArialRoundedMTBold28x31, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
													"%.2f   ",
													temperature);

				OLED_PrintXY(52, 90, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
															"%cC  ",0x80);
			}


			if (pressure_sensor_avaialble == true){
				OLED_PrintXY(2, 106, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
									"T: %.2f%cC  ",
											temperature, 0x80);

				OLED_PrintXY(2, 116, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true,
											"P: %.2fmbar  ",
													pressure);
			}
			OLED_UpdateScreen();
	    }

	    if ((sensor_count % PAS_CO2_SENSOR_UPDATE_COUNT) == 0)
	    {
	    	//printf("READ SUCCESS\r\n");

	    	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "-------------------------------------------");
	    	if(xensiv_pasco2_available == true){
	    		//printf("CO2:       : %8d ppm\r\n", ppm);
	    		EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "CO2:       : %8d ppm", ppm);
	    	}
	        /* Get the pressure and temperature data and print the results to the UART */
	    	if(pressure_sensor_avaialble == true){
				//printf("Pressure   : %8.2f mbar\r\n", pressure);
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Pressure   : %8.2f mbar", pressure);
				//printf("Temperature: %8.2f %cC\r\n\r\n", temperature, (char)176);
				//EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Temperature: %8.2f %cC", temperature, (char)176);
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Temperature: %8.2f *C", temperature);
	    	}

			EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "-------------------------------------------");
	    }

	    sensor_count++;
		vTaskDelay(pdMS_TO_TICKS(SENSOR_TASK_INTERVAL_MS));
	}
}
