/*
 * wifi_connection_task.c
 *
 *  Created on: 04.04.2023.
 *      Author: ilicb
 */

/* FreeRTOS header files */
#include "FreeRTOS.h"
#include "task.h"

#include "cyhal.h"
#include "cybsp.h"

/* Configuration file for Wi-Fi and MQTT client */
#include "wifi_config.h"

/* LwIP header files */
#include "lwip/netif.h"
#include "lwip/apps/sntp.h"

#include "iotconnect.h"
#include "iotconnect_common.h"
#include "iotc_mtb_time.h"
#include "iotconnect_telemetry.h"

#include "app_config.h"
#include "optiga_trust_task.h"

#include "drum_oled.h"
#include "drum_fonts.h"

#include "xensiv_pasco2_mtb.h"
#include "xensiv_dps3xx_mtb.h"

#include "cy_retarget_io.h"
#include "cy_rgb_led.h"

#include "ebv_logs.h"

#include "app_ver.h"

#include "iotc_eeprom.h"

extern char certificate[CERT_BUF_SIZE];
extern char IOTCONNECT_DUID[31];
extern char IOTCONNECT_CPID [32];
extern char IOTCONNECT_ENV  [8];
//extern xensiv_pasco2_t xensiv_pasco2;
//extern xensiv_dps3xx_t pressure_sensor;


#define IOT_CONN_RETRY_INTERVAL_MS       (5000)
#define DEFAULT_PRESSURE_REF_HPA    (0x3F7)     /* Default atmospheric pressure to compensate for (hPa) */


#define IOTC_MAIN_LOOP_INTERVAL_MS 			1000
#define IOTC_TELEMETRY_SEND_WAIT_LOOPS 		10

#define RGB_BRIGHTNESS_RESP 				"LED_RGB_Brightness "
#define RGB_BRIGHTNESS_RESP_LEN 			(17)

#define RGB_COLOR_RESP 				"LED_RGB_color "
#define RGB_COLOR_RESP_LEN 			(14)

uint16_t uLoopCount = 0;
uint8_t uIsAPConnected = 0;
uint8_t uLedRead = 1;
bool bLedRead;
bool bManualACK = false;
uint8_t uLED_RGB_Brightness = 0, uLED_RGB_Brightness_Last = 0;
uint32_t uLED_RGB_Color = CY_RGB_LED_COLOR_WHITE;


extern wifi_credentials_t wifi_credentials;
extern uint16_t ppm;
extern float pressure, temperature;
extern uint8_t xensiv_pasco2_available, pressure_sensor_avaialble;

cy_wcm_associated_ap_info_t iotc_ap_info;

//extern TaskHandle_t inputTaskHandle;
extern TaskHandle_t sensorTaskHandle;
//extern TaskHandle_t  wifiConnTaskHandle;

cy_wcm_connect_params_t connect_param;
cy_wcm_ip_address_t ip_address;

static const char *TAG = "wifi_connection_task";


void GUI_icon_signal_level(int rssi){
	int myrssi = - rssi;
	if(myrssi == 0){
		//SSD1327_PrintXY(90, 23, &Font_7x10, SSD1327_COLOR_WHITE, "W x  ", rssi);
		//OLED_PrintXY(90, 23, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "W x  ", rssi);
		OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "0");
		return;
	}
	if(myrssi > 100)	{
		OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "1", rssi);
		return;
	}
	if(myrssi > 90)	{
		OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "2", rssi);
		return;
	}
	if(myrssi > 80)	{
		OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "3", rssi);
		return;
	}
	if(myrssi > 70)	{
		OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "4", rssi);
		return;
	}
	OLED_PrintXY(110, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "5", rssi);
	return;
}

void GUI_icon_cloud(int cloud){
	switch (cloud){
	case 0:
		OLED_PrintXY(98, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "y");
		break;
	case 1:
		OLED_PrintXY(98, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "z");
		break;
	case 2:
		OLED_PrintXY(98, 23, &SmallSym10x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "\{");
		break;
	default:
		break;
	}
}


/******************************************************************************
 * Function Name: wifi_connect
 ******************************************************************************
 * Summary:
 *  Function that initiates connection to the Wi-Fi Access Point using the
 *  specified SSID and PASSWORD. The connection is retried a maximum of
 *  'MAX_WIFI_CONN_RETRIES' times with interval of 'WIFI_CONN_RETRY_INTERVAL_MS'
 *  milliseconds.
 *
 * Parameters:
 *  void
 *
 * Return:
 *  cy_rslt_t : CY_RSLT_SUCCESS upon a successful Wi-Fi connection, else an
 *              error code indicating the failure.
 *
 ******************************************************************************/
static cy_rslt_t wifi_connect(void) {
	cy_rslt_t result = CY_RSLT_SUCCESS;
    GUI_icon_cloud(0);
    GUI_icon_signal_level(0);
    //OLED_UpdateScreen();

    /* Check if Wi-Fi connection is already established. */
    if (cy_wcm_is_connected_to_ap() == 0) {
        /* Configure the connection parameters for the Wi-Fi interface. */
        memset(&connect_param, 0, sizeof(cy_wcm_connect_params_t));
        memcpy(connect_param.ap_credentials.SSID, wifi_credentials.sSSID, strlen(wifi_credentials.sSSID)+1);
        memcpy(connect_param.ap_credentials.password, wifi_credentials.sPWD, strlen(wifi_credentials.sPWD)+1);

        connect_param.ap_credentials.security = WIFI_SECURITY;

        //printf("Connecting to Wi-Fi AP '%s'\n", connect_param.ap_credentials.SSID);
        EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Connecting to Wi-Fi AP '%s'", connect_param.ap_credentials.SSID);

        /* Connect to the Wi-Fi AP. */
        for (uint32_t retry_count = 0; retry_count < MAX_WIFI_CONN_RETRIES; retry_count++) {

        	//in case new credentials entered
        	memcpy(connect_param.ap_credentials.SSID, wifi_credentials.sSSID, strlen(wifi_credentials.sSSID)+1);
        	memcpy(connect_param.ap_credentials.password, wifi_credentials.sPWD, strlen(wifi_credentials.sPWD)+1);
            result = cy_wcm_connect_ap(&connect_param, &ip_address);

            if (result == CY_RSLT_SUCCESS) {
                //printf("\nSuccessfully connected to Wi-Fi network '%s'.\n", connect_param.ap_credentials.SSID);
                EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Successfully connected to Wi-Fi network '%s'", connect_param.ap_credentials.SSID);

                cy_wcm_get_associated_ap_info(&iotc_ap_info);
                GUI_icon_signal_level(iotc_ap_info.signal_strength);
                //OLED_UpdateScreen();

                /* Set the appropriate bit in the status_flag to denote
                 * successful Wi-Fi connection, print the assigned IP address.
                 */
                if (ip_address.version == CY_WCM_IP_VER_V4) {
                    //printf("IPv4 Address Assigned: %s\n", ip4addr_ntoa((const ip4_addr_t*) &ip_address.ip.v4));
                    EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "IPv4 Address Assigned: %s", ip4addr_ntoa((const ip4_addr_t*) &ip_address.ip.v4));
                } else if (ip_address.version == CY_WCM_IP_VER_V6) {
                    //printf("IPv6 Address Assigned: %s\n", ip6addr_ntoa((const ip6_addr_t*) &ip_address.ip.v6));
                    EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "IPv6 Address Assigned: %s", ip6addr_ntoa((const ip6_addr_t*) &ip_address.ip.v6));
                }
                return result;
            }

            //printf("Connection to Wi-Fi network failed with error code 0x%0X. Retrying in %d ms. Retries left: %d\n",
            //        (int) result, WIFI_CONN_RETRY_INTERVAL_MS, (int) (MAX_WIFI_CONN_RETRIES - retry_count - 1));
            EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "Connection to Wi-Fi network failed with error code 0x%0X. Retrying in %d ms. Retries left: %d",
                       (int) result, WIFI_CONN_RETRY_INTERVAL_MS, (int) (MAX_WIFI_CONN_RETRIES - retry_count - 1));
            vTaskDelay(pdMS_TO_TICKS(WIFI_CONN_RETRY_INTERVAL_MS));
        }

        //printf("\nExceeded maximum Wi-Fi connection attempts!\n");
        EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "Exceeded maximum Wi-Fi connection attempts!");
        //printf("Wi-Fi connection failed after retrying for %d mins\n",
        //        (int) ((WIFI_CONN_RETRY_INTERVAL_MS * MAX_WIFI_CONN_RETRIES) / 60000u));
        EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "Wi-Fi connection failed after retrying for %d mins",
                (int) ((WIFI_CONN_RETRY_INTERVAL_MS * MAX_WIFI_CONN_RETRIES) / 60000u));
    }
    return result;
}

static void publish_telemetry() {
    IotclMessageHandle msg = iotcl_telemetry_create();
    char input_string_tmp[32];

    GUI_icon_cloud(2);
    //OLED_UpdateScreen();

    //printf("Preparing telemetry data: ");
    EBV_LOGS_D_CONT(TAG, EBV_LOGS_MODULE_DRIVER, "Preparing telemetry data: ");

    // Optional. The first time you create a data point, the current timestamp will be automatically added
    // TelemetryAddWith* calls are only required if sending multiple data points in one packet.
    //printf("time");
    EBV_LOGS_D_NEXT(TAG, EBV_LOGS_MODULE_DRIVER, "time");
    iotcl_telemetry_add_with_iso_time(msg, iotcl_iso_timestamp_now());

    //printf(", version");
    //EBV_LOGS_D_NEXT(TAG, EBV_LOGS_MODULE_DRIVER, ", version");
    iotcl_telemetry_set_string(msg, "version", APP_VERSION);
    iotcl_telemetry_set_number(msg, "cpu", 3.123); //

    //printf(", sensors");
    EBV_LOGS_D_NEXT(TAG, EBV_LOGS_MODULE_DRIVER, ", sensors");
    if(xensiv_pasco2_available == true) {
    	iotcl_telemetry_set_number(msg, "co2level", ppm); // co2 level data
    }

    if(pressure_sensor_avaialble == true){
		sprintf(input_string_tmp, "%.2f", pressure);
		iotcl_telemetry_set_string(msg, "pressure", input_string_tmp); // pressure data

		sprintf(input_string_tmp, "%.2f", temperature);
		iotcl_telemetry_set_string(msg, "temperature", input_string_tmp); // temperature data
	}

    bLedRead = ( Cy_GPIO_ReadOut(CYHAL_GET_PORTADDR(CYBSP_USER_LED), CYHAL_GET_PIN(CYBSP_USER_LED)) == 0);
    iotcl_telemetry_set_number(msg, "LED1", bLedRead); // LED1 status

    iotcl_telemetry_set_number(msg, "LED_RGB_Brightness", uLED_RGB_Brightness); //RGB brightness

    iotcl_telemetry_set_string(msg, "latitude", lat_lon_data.sLatitude);
    iotcl_telemetry_set_string(msg, "longitude", lat_lon_data.sLongitude);




    //printf(", creating serialized string");
    EBV_LOGS_D_NEXT(TAG, EBV_LOGS_MODULE_DRIVER, ", creating serialized string");
    const char *str = iotcl_create_serialized_string(msg, false);
    //printf(", cleaning up 'msg'.\n");
    EBV_LOGS_D_RESET(TAG, EBV_LOGS_MODULE_DRIVER, ", cleaning up 'msg'.");
    iotcl_telemetry_destroy(msg);
    //printf("Sending: %s\n\n", str);
    EBV_LOGS_D(TAG, EBV_LOGS_MODULE_DRIVER, "Sending: %s", str);
    cy_rslt_t result = iotconnect_sdk_send_packet(str); // underlying code will report an error
    if (CY_RSLT_SUCCESS != result) {
    	//printf("Error: iotconnect_sdk_send_packet failed!\n");
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "Error: iotconnect_sdk_send_packet failed!");
    	//result = cy_wcm_disconnect_ap();
    	result = iotconnect_sdk_disconnect();
    	if (CY_RSLT_SUCCESS != result){
    		//printf("Disconnecting iot connection failed\n");
    		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_sdk_disconnect: failed");
    	}
        vTaskSuspend(sensorTaskHandle);
        result = iotconnect_sdk_init();
    	if (CY_RSLT_SUCCESS != result){
    		//printf("iot connection init failed\n");
    		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_sdk_init: failed");
    	}
        vTaskResume(sensorTaskHandle);
    }
    iotcl_destroy_serialized(str);

    GUI_icon_cloud(1);
    //OLED_UpdateScreen();
}

/**
* \brief IoTConnect function to process command received from the cloud.
*/
static void iotconnect_on_cmd(IotclEventData data)
{
	int ptr = 13;
	int num = 0;
	int mul = 10;

	const char *command = iotcl_clone_command(data);
	//printf("\niotconnect_on_cmd: received command {%s}\r\n", command);
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_on_cmd: received command {%s}", command);
	/* Process commands. */
	if (!strcasecmp(command, "LED1 ON") || !strcasecmp(command, "LED1 1")) {
		//printf("iotconnect_on_cmd: LED1 ON");
		EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_on_cmd: LED1 ON");
		/* Invert the USER LED state */
		cyhal_gpio_write_internal(CYBSP_USER_LED, 0);
		uLedRead = 1;
	}
	else if (!strcasecmp(command, "LED1 OFF") || !strcasecmp(command, "LED1 0")) {
		//printf("iotconnect_on_cmd: LED1 OFF");
		EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_on_cmd: LED1 OFF");
		/* Invert the USER LED state */
		cyhal_gpio_write_internal(CYBSP_USER_LED, 1);
		uLedRead = 0;
	}

	//strncasecmp

	if(!strncmp(command, RGB_BRIGHTNESS_RESP, RGB_BRIGHTNESS_RESP_LEN)){
		ptr = RGB_BRIGHTNESS_RESP_LEN;
		while (command[ptr] != 0) {
			if((command[ptr] >= '0') & (command[ptr] <='9')){
			num = num * mul + (command[ptr] - 48);
				if (num > CY_RGB_LED_MAX_BRIGHTNESS) {
					/* Something went wrong - max value exceeded. */
					break;
				}
			}
			ptr++;
		}
		if((num >= 0) & (num <= CY_RGB_LED_MAX_BRIGHTNESS)){
			uLED_RGB_Brightness = num;
			if (uLED_RGB_Brightness == 0){
				cy_rgb_led_off();
			} else {
				cy_rgb_led_on(uLED_RGB_Color, uLED_RGB_Brightness);
			}
		}
	}
	if(!strncmp(command, RGB_COLOR_RESP, RGB_COLOR_RESP_LEN)){
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "RED")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_RED;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "GREEN")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_GREEN;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "BLUE")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_BLUE;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "WHITE")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_WHITE;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "YELLOW")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_YELLOW;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "CYAN")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_CYAN;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "PURPLE")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_PURPLE;
		}
		if (!strcasecmp(command+RGB_COLOR_RESP_LEN, "ORANGE")){
			uLED_RGB_Color = CY_RGB_LED_COLOR_RED + (CY_RGB_LED_COLOR_GREEN >> 4); //mixing Orange as full RED and half GREEN
		}

		cy_rgb_led_set_color(uLED_RGB_Color);
	}

	free((void *)command);

	bManualACK = true;

//	/* Send command acknowledge message to the cloud. */
//	const char *ack_json = iotcl_create_ack_string_and_destroy_event(data, true, NULL);
//	cy_rslt_t result = iotconnect_sdk_send_packet(ack_json); // underlying code will report an error
//	if (CY_RSLT_SUCCESS != result) {
//		printf("Error: iotconnect_sdk_send_packet failed!\n");
//	}
//	free((void *)ack_json);
}

void wifi_conn_task(void *pvParameters) {

    /* To avoid compiler warnings */
    (void) pvParameters;

    while (1)
    {
    	while(!cy_wcm_is_connected_to_ap()){
    	    /* Configure the Wi-Fi interface as a Wi-Fi STA (i.e. Client). */
    	    cy_wcm_config_t config = { .interface = CY_WCM_INTERFACE_TYPE_STA };

    	    /* Create a message queue to communicate with other tasks and callbacks. */
    	    //mqtt_task_q = xQueueCreate(MQTT_TASK_QUEUE_LENGTH, sizeof(mqtt_task_cmd_t));
    	    /* Initialize the Wi-Fi Connection Manager and jump to the cleanup block
    	     * upon failure.
    	     */
    	    if (CY_RSLT_SUCCESS != cy_wcm_init(&config)) {
    	        //printf("Error: Wi-Fi Connection Manager initialization failed!\n");
    	        EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "cy_wcm_init: failed. Error: Wi-Fi Connection Manager initialization failed");
    	        //goto exit_cleanup;
    	    }

    	    /* Set the appropriate bit in the status_flag to denote successful
    	     * WCM initialization.
    	     */
    	    //printf("Wi-Fi Connection Manager initialized.\n");
    	    EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Wi-Fi Connection Manager initialized.");

    		/* Initiate connection to the Wi-Fi AP and cleanup if the operation fails. */
    		if (CY_RSLT_SUCCESS != wifi_connect()) {
    			//printf("Error: Wi-Fi AP initialization failed!\n");
    			EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "wifi_connect: failed. Error: Wi-Fi AP initialization failed");
    		}

    		if ( sntp_enabled() ) {
    			sntp_stop();
    		}

    		if (0 != iotc_mtb_time_obtain(IOTCONNECT_SNTP_SERVER)) {
    			//printf("Error: iotc_mtb_time_obtain failed!\n");
    			EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "iotc_mtb_time_obtain: failed");
    		}
    	}
        IotConnectClientConfig *iotc_config = iotconnect_sdk_init_and_get_config();
        iotc_config->duid = IOTCONNECT_DUID;
        iotc_config->cpid = IOTCONNECT_CPID;
        iotc_config->env =  IOTCONNECT_ENV;
        iotc_config->auth.type = IOTCONNECT_AUTH_TYPE;

        if (iotc_config->auth.type == IOTC_AT_X509) {
            iotc_config->auth.data.cert_info.device_cert = (const char *)certificate;
            iotc_config->auth.data.cert_info.device_key = (const char *)DUMMY_PRIVATE_KEY;
        }

        iotc_config->cmd_cb = iotconnect_on_cmd;
        //printf("Initializing IoTConnect SDK...\n");
        EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Initializing IoTConnect SDK...");
        vTaskSuspend(sensorTaskHandle);
        cy_rslt_t ret = iotconnect_sdk_init();
        vTaskResume(sensorTaskHandle);

        if (CY_RSLT_SUCCESS != ret) {
            //printf("Failed to initialize the IoTConnect SDK. Error code: %lu\n", ret);
            EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "Failed to initialize the IoTConnect SDK. Error code: %lu", ret);
            //TO DO
            //goto exit_cleanup;
        } else {
        	//printf("Initializing IoTConnect SDK successful!\n");
        	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Initializing IoTConnect SDK successful!");



			uLoopCount = 0; //to send telemetry immediatelly
			//for (int j = 0; iotconnect_sdk_is_connected() && j < 3; j++) {
			while(1){
				vTaskSuspend(sensorTaskHandle);
				if(iotconnect_sdk_is_connected() != true) {
					vTaskResume(sensorTaskHandle);
					break;
				}
				vTaskResume(sensorTaskHandle);

				uIsAPConnected = cy_wcm_is_connected_to_ap();
				if(!uIsAPConnected) {
					GUI_icon_signal_level(0);
					break;
				}
				cy_wcm_get_associated_ap_info(&iotc_ap_info);
				GUI_icon_signal_level(iotc_ap_info.signal_strength);
				//OLED_UpdateScreen(); done within sensor loop

				if(((uLoopCount % IOTC_TELEMETRY_SEND_WAIT_LOOPS) == 0) | (bManualACK == true)){
					//printf("Publishing data\n");
					EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Publishing data");
					publish_telemetry();
					bManualACK = false;
					uLoopCount = 0; //reset loop count to wait for full publish cycle
				}

				uLoopCount++;
				vTaskDelay(pdMS_TO_TICKS(IOTC_MAIN_LOOP_INTERVAL_MS));
			}


			//printf("Disconnecting IoTConnect SDK...");
			EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "Disconnecting IoTConnect SDK...");
			if (CY_RSLT_SUCCESS == iotconnect_sdk_disconnect()) {
				//printf("OK\n");
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_DRIVER, "OK");
			} else {
				EBV_LOGS_E(TAG, EBV_LOGS_MODULE_DRIVER, "iotconnect_sdk_disconnect: failed");
			}
			vTaskDelay(pdMS_TO_TICKS(IOT_CONN_RETRY_INTERVAL_MS));
        }
    }
}



