package com.avnet.ebv.viewmodel

import android.content.Context
import android.content.Intent
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.request.AddDeleteDeviceEntityRequest
import com.avnet.ebv.model.request.AddDeviceRequest
import com.avnet.ebv.model.request.GrantDevicePermissionRequest
import com.avnet.ebv.model.request.UpdateDeviceEntityRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.view.activity.MainActivity
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class AddDeviceViewModel @Inject
constructor(
    private val dataRepositoryRepository: DeviceRepositorySource,
    private val deviceRepository: DeviceRepositorySource
) : BaseViewModel() {
    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    val addDeviceResponse: LiveData<Resource<AddDeviceResponse>> get() = deviceResponse
    private var deviceResponse = MutableLiveData<Resource<AddDeviceResponse>>()

    val getUserIdentityResponse: LiveData<Resource<UserIdentityResponse>> get() = _userIdentity
    private var _userIdentity = MutableLiveData<Resource<UserIdentityResponse>>()

    var _templateResponse = MutableLiveData<MainDeviceTemplateResponse>()

    var _singleDeviceDetailResponse = MutableLiveData<SingleDeviceDetailResponse>()

    var _addResponse = MutableLiveData<DeviceUpdateResponse>()

    var _deviceAllocatedDetailsResponse = MutableLiveData<GetDeviceAllocatedDetailsResponse>()
    var _permissionsGrantedDeviceResponse = MutableLiveData<DeviceGrantedPermissionsResponse>()


    var deviceId: MutableLiveData<String> = MutableLiveData()
    var deviceName: MutableLiveData<String> = MutableLiveData()
    var entityGuid: MutableLiveData<String> = MutableLiveData()
    var deviceTemplateGuid: MutableLiveData<String> = MutableLiveData()
    var userDetails: UserIdentityResponse? = null

    init {
        deviceId.value = ""
        deviceName.value = ""
        entityGuid.value = ""
        deviceTemplateGuid.value = ""
    }

    fun initializeWithContext(context: Context) {
        this.context = context
        userDetails = GsonBuilder().create().fromJson(
            SharedPref.instance!!.getValueFromPreferences(context.getString(R.string.pref_user_details)),
            UserIdentityResponse::class.java
        )
    }

    fun addDeviceResponse(addDeviceRequest: AddDeviceRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                dataRepositoryRepository.addDevice(addDeviceRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    deviceResponse.value = it
                    if (deviceResponse.value!!.data?.status == AppConstants.STATUS_200) {

                        if ((context as MainActivity).navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
                        {
                            (context as MainActivity).navController.navigateUp()
                        } else {
                            val inflater = (context as MainActivity).navController.navInflater
                            var graph = inflater.inflate(R.navigation.navigation)

                            (context as MainActivity).navController.setGraph(
                                graph,
                                (context as MainActivity).intent.extras
                            )
                        }

                        showMsg(it.data?.message.toString())

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }

                    AndroidLog.d(
                        "Add Device ::::::::::::::::    ",
                        deviceResponse.value!!.data?.error?.get(0)?.message.toString()
                    )
                }

            }
        }
    }

    fun getUserIdentity() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                dataRepositoryRepository.requestGetUserIdentity().collect() {
                    _userIdentity.value = it
                    if (_userIdentity.value!!.data?.status == AppConstants.STATUS_200) {
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_user_details),
                            Gson().toJson(it.data)
                        )
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_entity_guid),
                            it.data?.data?.entityGuid.toString()
                        )
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_company_guid),
                            it.data?.data?.companyGuid.toString()
                        )
                        entityGuid.value = it.data?.data?.entityGuid.toString()
//                        deviceTemplateGuid.value = it.data?.data!!.toString()

                        val intent = Intent(AppConstants.INTENT_USER_DETAILS)
                        (context as BaseActivity).sendBroadcast(intent)

//                        showMsg(it.data?.message.toString())
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun getTemplate() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                _ProgreeBarState.value = ProgressState.SHOW
                dataRepositoryRepository.requestDeviceTemplateList1()
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
                            _templateResponse.value = it.data!!
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getSingleDeviceDetial(uniqueId: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                _ProgreeBarState.value = ProgressState.SHOW
                deviceRepository.getSingleDeviceDetail(uniqueId)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            _singleDeviceDetailResponse.value = it.data!!


                            getDeviceAllocatedList(it.data.data.get(0).guid)

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getDeviceAllocatedList(deviceGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {

                deviceRepository.getDeviceGrantDetailsApi(deviceGuid)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            _deviceAllocatedDetailsResponse.value = it.data!!
                            if (it.data.data.size == 0) {
                                addPermissionsDeviceDetails(deviceGuid)
                            } else {
                                if (it.data.data.get(0).userGuid.equals(userDetails?.data?.userGuid)){

                                    showMsg(context.getString(R.string.alert_failed_to_add_device_with_same_user))
                                }else{
                                    showMsg(context.getString(R.string.alert_failed_to_add_device))
                                }

                            }

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun addPermissionsDeviceDetails(deviceGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {


                val list: ArrayList<String>
                list = ArrayList()
                list.add(userDetails?.data?.userId.toString())
                val requestGrantPermission = GrantDevicePermissionRequest(
                    list,
                    GrantDevicePermissionRequest.Permission(true, true)
                )

                deviceRepository.addPermissionsDeviceDetails(deviceGuid, requestGrantPermission)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            _permissionsGrantedDeviceResponse.value = it.data!!

                            addDevice(
                                deviceGuid,
                                SharedPref.instance!!.getValueFromPreferences(context.getString(R.string.pref_entity_guid))
                                    .toString()
                            )

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun addDevice(deviceGuid: String, entityGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                var request = AddDeleteDeviceEntityRequest()
                var requestItem =
                    AddDeleteDeviceEntityRequest.AddDeleteDeviceEntityRequestItem(deviceGuid)
                request.add(requestItem)

                deviceRepository.addDeviceEntity(entityGuid, request)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
                            _addResponse.value = it.data!!

                            updateDevice(deviceGuid,entityGuid)

//                            showMsg(context.getString(R.string.text_device_added_successfully))


                        } else if (it.errorBody?.status == AppConstants.STATUS_412) {
                            showError(it.errorBody)
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun updateDevice(deviceGuid: String, entityGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                var request = UpdateDeviceEntityRequest(deviceName.value.toString(),entityGuid)

                deviceRepository.updateDeviceEntity(deviceGuid, request)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
//                            _addResponse.value = it.data!!
                            if ((context as MainActivity).navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
                            {
                                (context as MainActivity).navController.navigateUp()
                            } else {
                                val inflater = (context as MainActivity).navController.navInflater
                                var graph = inflater.inflate(R.navigation.navigation)

                                (context as MainActivity).navController.setGraph(
                                    graph,
                                    (context as MainActivity).intent.extras
                                )
                            }

                            showMsg(context.getString(R.string.text_device_added_successfully))


                        } else if (it.errorBody?.status == AppConstants.STATUS_412) {
                            showError(it.errorBody)
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun checkIsValid(): Boolean {
        return if (deviceId.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_device_identifier))
            false
        }/* else if (deviceName.value.equals("", ignoreCase = true)) {
            Toast.makeText(
                context,
                context.getString(R.string.alert_enter_friendly_device_name),
                Toast.LENGTH_LONG
            ).show()
            false
        }*//*else if (deviceTemplateGuid.value.equals("", ignoreCase = true)) {
            Toast.makeText(
                context,
                context.getString(R.string.alert_enter_deviceGuid),
                Toast.LENGTH_LONG
            ).show()
            false
        }*/ else {
            true
        }
    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        _ProgreeBarState.value = ProgressState.HIDE
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }

}
