package com.avnet.ebv.viewmodel

import android.content.Context
import android.content.Intent
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.AddDeleteDeviceEntityRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.view.adater.DeviceListAdapter
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class DeviceListViewModel @Inject
constructor(
    private val deviceRepository: DeviceRepositorySource
) : BaseViewModel() {

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    val getUserIdentityResponse: LiveData<Resource<UserIdentityResponse>> get() = _userIdentity
    private var _userIdentity = MutableLiveData<Resource<UserIdentityResponse>>()

    var deviceResponse = MutableLiveData<DeviceListResponse>()

    var deviceList = ArrayList<DeviceListResponse.Data?>()

    var _templateResponse = MutableLiveData<MainDeviceTemplateResponse>()
    var _deleteResponse = MutableLiveData<DeviceUpdateResponse>()

    var _deviceAllocatedDetailsResponse = MutableLiveData<GetDeviceAllocatedDetailsResponse>()
    var _revokeDeviceAllocatedPermissionsResponse =
        MutableLiveData<RevokeDevicePermissionsResponse>()

    var _pageNumber = MutableLiveData<Int>()
    var _pageSize = MutableLiveData<String>()
    var _totalDeviceCount = MutableLiveData<Int>()
    var isApiCalled: Boolean = false
    var callPagination: Boolean = false
    var userDetails: UserIdentityResponse? = null

    init {
        _pageNumber.value = 1
        _pageSize.value = "20"
        _totalDeviceCount.value = 0
        deviceList = ArrayList()
        _totalDeviceCount.value = -1
    }

    fun initializeWithContext(context: Context) {
        this.context = context
        userDetails = GsonBuilder().create().fromJson(
            SharedPref.instance!!.getValueFromPreferences(context.getString(R.string.pref_user_details)),
            UserIdentityResponse::class.java
        )
    }

    fun getDeviceList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = _pageNumber.value.toString()
                params[ApiConstant.pageSize] = _pageSize.value

                deviceRepository.requestDeviceList(params).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        deviceResponse.value = it.data!!
                        _totalDeviceCount.value = it.data.count!!
                        if (!it.data.count!!.equals(deviceList.size)) {
                            _pageNumber.value = _pageNumber.value!!.plus(1)
                            callPagination = true
                            isApiCalled = false
                        } else {
                            callPagination = false
                        }

                    } else if (it.errorBody?.status == AppConstants.STATUS_204) {
                        if (deviceList.size == 0) {
                            _totalDeviceCount.value = 0
                        }
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getUserIdentity() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                deviceRepository.requestGetUserIdentity().collect() {
                    _userIdentity.value = it
                    if (_userIdentity.value!!.data?.status == AppConstants.STATUS_200) {
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_user_details),
                            Gson().toJson(it.data)
                        )

                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_entity_guid),
                            it.data?.data?.entityGuid.toString()
                        )
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_company_guid),
                            it.data?.data?.companyGuid.toString()
                        )
                        val intent = Intent(AppConstants.INTENT_USER_DETAILS)
                        (context as BaseActivity).sendBroadcast(intent)

//                        showMsg(it.data?.message.toString())
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun getTemplate() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {

                deviceRepository.requestDeviceTemplateList1()
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
                            _templateResponse.value = it.data!!
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getDeviceAllocatedList(
        adapter: DeviceListAdapter,
        position: Int,
        deviceGuid: String,
        entityGuid: String
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {

                deviceRepository.getDeviceGrantDetailsApi(deviceGuid)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            _deviceAllocatedDetailsResponse.value = it.data!!
                            if (it.data.data.size > 0) {
                                if (userDetails?.data?.userGuid.equals(
                                        it.data.data.get(0).userGuid,
                                        true
                                    )
                                ) {
                                    revokeAlocatedPermissionsApi(
                                        adapter,
                                        position,
                                        deviceGuid,
                                        entityGuid,
                                        it.data.data.get(0).userDevicePermissionGuid
                                    )
                                } else {
                                    showMsg(context.getString(R.string.alert_failed_device_not_present_with_you))
                                }
                            } else {
                                //todo remove this call ...
                                deleteDevice(adapter, position, deviceGuid, entityGuid)
                            }


                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun revokeAlocatedPermissionsApi(
        adapter: DeviceListAdapter,
        position: Int,
        deviceGuid: String,
        entityGuid: String, userDevicePermissionGuid: String
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {

                deviceRepository.revokeAlocatedPermissionsApi(userDevicePermissionGuid)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            _revokeDeviceAllocatedPermissionsResponse.value = it.data!!
                            deleteDevice(adapter, position, deviceGuid, entityGuid)


                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun deleteDevice(
        adapter: DeviceListAdapter,
        position: Int,
        deviceGuid: String,
        entityGuid: String
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                var request = AddDeleteDeviceEntityRequest()
                var requestItem =
                    AddDeleteDeviceEntityRequest.AddDeleteDeviceEntityRequestItem(deviceGuid)
                request.add(requestItem)

                deviceRepository.deleteDeviceEntity(entityGuid, request)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
                            _deleteResponse.value = it.data!!
//                            showMsg(it.data.message)
                            deviceList.remove(deviceList.get(position))
                            adapter.notifyDataSetChanged()
                            adapter.viewBinderHelper.closeLayout(position.toString())

                            if (deviceList.size == 0) {
                                _totalDeviceCount.value = 0
                            }

                        } else if (it.errorBody?.status == AppConstants.STATUS_412) {
                            adapter.viewBinderHelper.closeLayout(position.toString())
                            if (it.errorBody.error != null && it.errorBody.error!!.size > 0) {
                                showMsg(it.errorBody.error!!.get(0)?.message.toString())
                            } else {
                                showMsg(it.errorBody.message.toString())
                            }
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            adapter.viewBinderHelper.closeLayout(position.toString())
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }
}
