package com.avnet.ebv.viewmodel

import android.content.Context
import android.content.Intent
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.*
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.request.ExecutePropertyRequest
import com.avnet.ebv.model.request.TwinPropertyRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants

import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import com.avnet.ebv.view.activity.DeviceDetailActivity
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject

class PropertiesViewModel @Inject
constructor(private val deviceRepository: DeviceRepositorySource) : BaseViewModel() {

    lateinit var context: Context
    var twinPropertyLookup = MutableLiveData<Resource<PropertyLookupResponse>>()
    var twinPropertyHistory = MutableLiveData<Resource<PropertyHistoryResponse>>()
    var desiredValue: MutableLiveData<String> = MutableLiveData()
    var twinProperty: MutableLiveData<String> = MutableLiveData()
    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    var executeProperty = MutableLiveData<Resource<ExecuteCommandResponse>>()

    var deviceInfo: MutableLiveData<DeviceListResponse.Data?> = MutableLiveData()


    init {
        desiredValue.value = ""
        twinProperty.value = ""
    }

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    fun initializeWithContext(context: Context) {
        this.context = context
    }


    fun getPropertyLookList(templateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.HIDE
            viewModelScope.launch {
                deviceRepository.propertyLookupList(templateGuid).collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        twinPropertyLookup.value = it


                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }


    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun getPropertyHistoryList(deviceName: String) {

        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.HIDE
            viewModelScope.launch {
                deviceRepository.propertyHistoryList(deviceName).collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        twinPropertyHistory.value = it
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun checkIsValid(): Boolean {
        return if (desiredValue.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_device_desired_value))
            false
        } else {
            true
        }
    }

    fun executePropertyApi(uniqueId: String, executePropertyRequest: ExecutePropertyRequest) {

        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.executeProperty(uniqueId, executePropertyRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    executeProperty.value = it
                    if (executeProperty.value!!.data?.status == AppConstants.STATUS_200) {
                        desiredValue.value = ""

                        getPropertyHistoryList(deviceInfo.value!!.uniqueId.toString())
                        showMsg(it.data?.message.toString())

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }

                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }
}
